import React, { useState, useEffect, useReducer } from 'react';
import styles from './ContractContent.less';
import {
  Input,
  Form,
  message,
  Upload,
  Button,
  DatePicker,
  Pagination,
  Modal,
  Spin,
  Image,
} from 'antd';
const { TextArea } = Input;
import { PlusOutlined, LeftOutlined, LoadingOutlined } from '@ant-design/icons';
import { connect } from 'umi';

import LINE from '@/components/Line/Line';
import moment from 'moment';
import './ContractContent.less';

import SearchOptionsCommnity from '@/components/SearchOptions/SearchOptionsCommnity';
import { tipList } from '@/utils/tip';

import FileViewer from 'react-file-viewer';
import PDF from 'react-pdf-js';

import { randomString } from '@/utils/string';

/******* useState 改造 useReducer   *********/

// 初始值
const contentState = {
  fileList: [], // 图片列表
  uploadUp: true, // 禁止上传 直到选择了小区后
  comtyName: null, // 已选小区名字
  tipTime: ['previous month', 'two months'], //时间提示
  tipModal: false, //附件弹窗
  fileInfo: null, //附件内容
  numPages: 1, // pdf 总页码数
  pageNumber: 1, //  pdf 当前页码数
  imgInfo: null, //本地图片预览信息
  previewVisible: false, //本地图片预览弹窗
  imgLoad: false, //本地图片上传加载
};
// 方法
const contentReducer = (state, action) => {
  switch (action.type) {
    case 'setFileList': // 改变图片列表
      return { ...state, fileList: action.payload };

    case 'setUploadUp': // 附件上传限制
      return { ...state, uploadUp: action.payload };

    case 'setComtyName': // 小区名字
      return { ...state, comtyName: action.payload };

    case 'setTipTime': // 时间提示
      return { ...state, tipTime: action.payload };

    case 'settipModal': // 附件弹窗
      return { ...state, tipModal: action.payload };

    case 'setfileInfo': // 附件内容
      return { ...state, fileInfo: action.payload };

    case 'setNumPages': // PDF总页码数
      return { ...state, numPages: action.payload };

    case 'setPageNumber': // PDF当前页码数
      return { ...state, pageNumber: action.payload };

    case 'setimgInfo': // 本地图片预览信息
      return { ...state, imgInfo: action.payload };

    case 'setPreviewVisible': // 本地图片预览弹窗
      return { ...state, previewVisible: action.payload };

    case 'setimgLoad': // 本地图片上传加载
      return { ...state, imgLoad: action.payload };

    default:
      break;
  }
};

/******* useState 改造 useReducer   *********/

const ContractContent = (props: any) => {
  const { Contract, dispatch, FileImg, loading } = props;

  const { DataSaveDetail } = Contract;
  const { RangePicker } = DatePicker; // 日期组件
  const [form] = Form.useForm(); // 表单
  const RA = (index: any, values: any) => {
    dispatch({ type: 'Contract/RA', playload: { index: index, body: values } });
  };

  /* 使用 useReducer  */
  const [state, dispatchs] = useReducer(contentReducer, contentState);

  useEffect(() => {
    // 如果是添加传来没有值的时候 就清空  否则  赋值给表单
    if (DataSaveDetail == null) {
      // 清空表单
      form.resetFields();
    } else {
      // 发起图片请求
      let objData = {
        type: 'tosContractPreview',
        fileName: DataSaveDetail.contractFileName,
        extends: DataSaveDetail.communityName,
      };
      RA(47, objData);

      // 打开禁止
      // setUploadUp(false);
      dispatchs({ type: 'setUploadUp', payload: false });

      // 提示时间
      let a1 = moment(DataSaveDetail.contractValidEndDate)
        .subtract(2, 'month')
        .format('YYYY-MM-DD');
      let a2 = moment(DataSaveDetail.contractValidEndDate)
        .subtract(1, 'month')
        .format('YYYY-MM-DD');

      // setTipTime([a1, a2]);
      dispatchs({ type: 'setTipTime', payload: [a1, a2] });

      // 给到上传绑定
      // setComtyName(DataSaveDetail.communityName);
      dispatchs({ type: 'setComtyName', payload: DataSaveDetail.communityName });
      // 表单内容
      form.setFieldsValue({
        ...DataSaveDetail,
        upload: 'ok',
        time: [
          moment(DataSaveDetail.contractValidStartDate),
          moment(DataSaveDetail.contractValidEndDate),
        ],
      });
    }
  }, [DataSaveDetail]);

  // 监听上传图片列表
  useEffect(() => {
    if (FileImg != null) {
      let obj = new Array();
      for (var i in FileImg) {
        let a = {
          uid: i,
          name: FileImg[i].fileName,
          status: 'done',
          type: FileImg[i].fileName.match(/\.([^\.]+)$/)[1].toLowerCase(),
          url: FileImg[i].fileUrl,
        };
        console.log();
        obj.push(a);
      }
      // setFileList([...obj]);
      dispatchs({ type: 'setFileList', payload: [...obj] });
    } else {
      // setFileList([]);
      dispatchs({ type: 'setFileList', payload: [] });
    }
  }, [FileImg]);

  // 监听列表
  useEffect(() => {
    if (state.fileList.length == 0) {
      form.setFieldsValue['upload'] = null;
    }
  }, [state.fileList]);

  // 返回
  const goToReturn = () => {
    // console.log(fileList)
    history.back();
  };

  // 提交
  const onFinish = (values: any) => {
    // 判断有没有文件
    if (state.fileList.length == 0) {
      message.error('Please upload the attachment!');
    } else {
      values.contractValidStartDate = values.time[0].format('YYYY-MM-DD');
      values.contractValidEndDate = values.time[1].format('YYYY-MM-DD');
      let data = new Array();
      for (let i = 0; i < state.fileList.length; i++) {
        data.push(state.fileList[i].name);
      }
      values.contractFileNameList = data;

      // 编辑
      if (DataSaveDetail != null) {
        values.id = DataSaveDetail.id;
      }

      RA(27, values);
      // RA(27, values)
    }
  };

  // 上传文件设置
  const uploadProps = {
    accept: '.docx,.jpg,.png,.pdf',
    action: '/tos/image/upload',
    data: { imageType: 'tosContract', extends: state.comtyName },
    fileList: state.fileList,
    onChange: ({ file }: { file: any }) => {
      console.log(file);
      if (file.status === 'uploading') {
        // setimgLoad(true);
        dispatchs({ type: 'setimgLoad', payload: true });
      }
      if (file.status === 'error') {
        message.error(file.name + ' Upload  failed !');
        // setimgLoad(false);
        dispatchs({ type: 'setimgLoad', payload: false });
      }
      if (file.status === 'done') {
        message.success(file.name + ' Upload Successful !');

        // 实现在线预览
        let obj = {
          uid: randomString(12),
          name: file.name,
          status: 'done',
          type: file.name.match(/\.([^\.]+)$/)[1].toLowerCase(),
          url: file.response.data,
        };

        // setFileList([...fileList, obj]);
        dispatchs({ type: 'setFileList', payload: [...state.fileList, obj] });
        // 添加到表单
        form.setFieldsValue({
          upload: 'ok',
        });
        // setimgLoad(false);
        dispatchs({ type: 'setimgLoad', payload: false });
      }
    },
  };

  // // 设置提示倒计时
  const changeTime = (data: any, dateStrings: any) => {
    if (data != null) {
      let a1 = moment(data[1]).subtract(2, 'month').format('YYYY-MM-DD');
      let a2 = moment(data[1]).subtract(1, 'month').format('YYYY-MM-DD');
      // setTipTime([a1, a2]);
      dispatchs({ type: 'setTipTime', payload: [a1, a2] });
    }
  };

  // 选择小区名字并赋值
  const opname = (value: any) => {
    // 打开上传  如果选择了 小区就打开 否则  禁止上传
    value
      ? dispatchs({ type: 'setUploadUp', payload: false })
      : dispatchs({ type: 'setUploadUp', payload: true });
    // 如果切换了  清掉上传的图片
    if (value != state.comtyName) {
      // setFileList([]);
      dispatchs({ type: 'setFileList', payload: [] });
    }
    // setComtyName(value); // 赋值小区名字 给到上传
    dispatchs({ type: 'setComtyName', payload: value });

    form.setFieldsValue({
      communityName: value,
    });
  };

  //本地图预览处理
  function getBase64(file: any) {
    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      reader.readAsDataURL(file);
      reader.onload = () => resolve(reader.result);
      reader.onerror = (error) => reject(error);
    });
  }

  //点击预览
  const onPreviews = async (file: any) => {
    // 有值的话  就是本地预览
    if (file.size) {
      if (!file.url && !file.preview) {
        file.preview = await getBase64(file.originFileObj);
      }
      let info = {
        previewImage: file.url || file.preview,
        previewTitle: file.name || file.url.substring(file.url.lastIndexOf('/') + 1),
      };
      // setPreviewVisible(true); // 弹窗
      dispatchs({ type: 'setPreviewVisible', payload: true });
      // setimgInfo(info); // 图片信息
      dispatchs({ type: 'setimgInfo', payload: info });
    } else {
      // setfileInfo(file); // 设置选择的文件
      // settipModal(true); // 打开弹窗
      dispatchs({ type: 'setfileInfo', payload: file });
      dispatchs({ type: 'settipModal', payload: true });
    }
  };

  // 关闭预览弹窗
  const handleCancel = () => {
    // settipModal(false);
    dispatchs({ type: 'settipModal', payload: false });
  };

  // pdf 总页数
  const onDocumentLoadSuccess = (pages: any) => {
    // setNumPages(pages);
    dispatchs({ type: 'setNumPages', payload: pages });
  };

  // const onDocumentLoadSuccess = ({ numPages }: any) => {
  //   //numPages是总页数
  //   setNumPages(numPages);
  // };

  // 切换pdf 总数
  const onChangePage = (page: any) => {
    // setPageNumber(page);
    dispatchs({ type: 'setPageNumber', payload: page });
  };

  // 文件上传判断
  function beforeUpload(file: any) {
    // 文件大小判断
    const isLt2M = file.size / 1024 / 1024 < 2;
    if (!isLt2M) {
      message.error('File must be less than or equal to 2MB!');
    }
    return isLt2M;
  }

  // 移除文件
  const onRemove = async (file: any) => {
    let fileListArr = state.fileList;
    for (let i in fileListArr) {
      if (fileListArr[i].uid == file.uid) {
        fileListArr.splice(i, 1);
      }
    }
    // setFileList([...fileListArr]);
    dispatchs({ type: 'setFileList', payload: [...fileListArr] });
  };

  // 文件上传按钮
  const uploadButton = (
    <div>
      {state.imgLoad ? <LoadingOutlined /> : <PlusOutlined />}
      <div style={{ marginTop: 5 }}>Upload</div>
    </div>
  );

  return (
    <>
      <Spin spinning={loading}>
        <div className={styles.base}>
          {/* 头部组件 */}
          <div className={styles.box}>
            <div className={styles.item1}>{DataSaveDetail ? 'Edit' : 'Add'} Contract</div>
            <button className={styles.item3} onClick={goToReturn}>
              <LeftOutlined />
              Back
            </button>
          </div>
          {/* 表单  initialValues={defForm} */}
          <Form
            // ref={formRef}
            autoComplete="off"
            form={form}
            name="basic"
            initialValues={DataSaveDetail}
            onFinish={onFinish}
            // initialValues={defForm}
          >
            <div className="contract_box">
              <div className="list2">
                <Form.Item
                  name="contractNumber"
                  label="Contract Number"
                  rules={[{ required: true, message: `${tipList[0]}` }]}
                >
                  <Input style={{ width: 200 }} placeholder="Contract Number" />
                </Form.Item>
              </div>
              {/* 合同编号、甲方、乙方 */}
              <div className="list_1">
                <div className="item_1">
                  <Form.Item
                    name="contractPartyA"
                    label="Contract Party A"
                    rules={[{ required: true, message: `${tipList[1]}` }]}
                  >
                    <Input style={{ width: 300 }} placeholder="Contract Party A" />
                  </Form.Item>
                </div>
                <div className="item_1">
                  <Form.Item
                    name="contractPartyB"
                    label="Contract Party B"
                    rules={[{ required: true, message: `${tipList[2]}` }]}
                  >
                    <Input style={{ width: 300 }} placeholder="Contract Party B" />
                  </Form.Item>
                </div>
              </div>
              {/* 小区 */}
              <div className="list2">
                <div className="main">
                  <Form.Item
                    labelAlign="right"
                    name="communityName"
                    label="Community Name"
                    rules={[{ required: true, message: `${tipList[3]}` }]}
                  >
                    <SearchOptionsCommnity
                      defaultName={DataSaveDetail ? DataSaveDetail.communityName : null}
                      // ubmit={extendName}
                      opname={opname}
                    />
                  </Form.Item>
                </div>
              </div>
              {/* 合同标题 */}
              <div className="list2">
                <Form.Item
                  labelAlign="right"
                  name="contractTitle"
                  label="Contract Title"
                  rules={[{ required: true, message: `${tipList[4]}` }]}
                >
                  <Input style={{ width: 500 }} placeholder="Contract Title" />
                </Form.Item>
              </div>

              {/* 起止时间 */}
              <div className="list2">
                <Form.Item
                  name="time"
                  labelAlign="right"
                  label="Contract Time"
                  rules={[{ required: true, message: `${tipList[5]}` }]}
                >
                  <RangePicker
                    defaultValue={DataSaveDetail ? DataSaveDetail.time : null}
                    // disabledDate={disabledDate}   // 时间限制
                    placeholder={['Effective Date', 'Expiration Date']}
                    onChange={changeTime}
                  />
                </Form.Item>
              </div>

              {/* 附件上传 */}
              <div className="list2">
                <Form.Item
                  labelAlign="right"
                  name="upload"
                  label="Contract Annex"
                  rules={[{ required: true, message: `Please upload the attachment!` }]}
                >
                  <Upload
                    {...uploadProps}
                    listType={'picture-card'}
                    disabled={state.uploadUp}
                    beforeUpload={beforeUpload}
                    onPreview={onPreviews}
                    onRemove={onRemove}
                  >
                    {state.fileList.length >= 3 ? null : uploadButton}
                  </Upload>
                </Form.Item>
              </div>

              {/* 合同内容 */}
              <div className="list2">
                <div className={styles.box8item2x1}>
                  <Form.Item
                    rules={[{ required: true, message: `${tipList[5]}` }]}
                    label="Contract Remarks"
                    name="contractRemindContent"
                    style={{ marginBottom: 0 }}
                  >
                    <TextArea
                      style={{ padding: 8, width: 600 }}
                      placeholder="Remarks"
                      showCount={true}
                      autoSize={{ minRows: 6, maxRows: 6 }}
                    ></TextArea>
                  </Form.Item>
                </div>
                <div
                  className={styles.box8item2x2}
                  style={{ marginLeft: 150, textTransform: 'capitalize' }}
                >
                  <p style={{ marginBottom: 0 }}>
                    The system will send e-mail notification in the{' '}
                    <span style={{ color: 'red' }}>{state.tipTime[0]}</span> and{' '}
                    <span style={{ color: 'red' }}>{state.tipTime[1]}</span> that the contract is
                    about to expire !
                  </p>
                  <p style={{ marginBottom: 0 }}>
                    <span style={{ color: 'red' }}>*</span> If the time has passed, it will not be
                    sent
                  </p>
                  {/* <span style={{color:'#f00'}}>&nbsp;{stateTime}</span> */}
                </div>
              </div>

              <div>
                <LINE />
                <Button type="primary" htmlType="submit" loading={loading}>
                  Submit
                </Button>
              </div>
            </div>
          </Form>

          {/* 附件预览 */}
          <Modal
            title={state.fileInfo != null ? state.fileInfo.name : 'File Preview'}
            visible={state.tipModal}
            width={800}
            onCancel={handleCancel}
            style={{ textAlign: 'center' }}
            footer={null}
          >
            {state.fileInfo != null ? (
              state.fileInfo.type == 'pdf' ? (
                // pdf 换一种
                <>
                  <PDF
                    file={state.fileInfo.url}
                    // file={stringSplit(fileInfo.url, 'm/cash')}
                    page={state.pageNumber}
                    scale={1.25}
                    onDocumentComplete={onDocumentLoadSuccess}
                  />
                  <Pagination
                    style={{ textAlign: 'center', marginTop: 10, display: 'block' }}
                    simple
                    defaultCurrent={state.pageNumber}
                    total={state.numPages * 10}
                    onChange={onChangePage}
                  />
                </>
              ) : state.fileInfo.type == 'jpg' || state.fileInfo.type == 'png' ? (
                // 图片用指定格式

                <Image
                  src={state.fileInfo.url}
                  preview={false}
                  style={{ margin: '0 auto', textAlign: 'center' }}
                />
              ) : (
                <FileViewer
                  fileType={state.fileInfo.type}
                  filePath={state.fileInfo.url}
                  // filePath={stringSplit(fileInfo.url, 'm/cash')}
                />
              )
            ) : (
              ''
            )}
          </Modal>

          {/* 本地附件预览 */}
          {state.imgInfo != null ? (
            <Modal
              visible={state.previewVisible}
              title={state.imgInfo.previewTitle}
              footer={null}
              onCancel={() => {
                // setPreviewVisible(false);
                dispatchs({ type: 'setPreviewVisible', payload: false });
              }}
            >
              <img alt="example" style={{ width: '100%' }} src={state.imgInfo.previewImage} />
            </Modal>
          ) : (
            ''
          )}
        </div>
      </Spin>
    </>
  );
};

function mapStateToProps(state: any) {
  const { ContractModel, Contract } = state;
  const FileImg = Contract.FileImg;
  const loading = state.loading.models.Contract || false;
  return {
    ContractModel,
    Contract,
    FileImg,
    loading,
  };
}
export default connect(mapStateToProps)(ContractContent);
